/*
 *  cgi2perl.c
 *
 *  CGI Script which calls perl with the name of the script
 *  in PATH_INFO.
 *  based on:
 *
 *  egscrip2.c
 *
 *  Example CGI Script file for use with HTTPS.  Assumes it is invoked from a form.
 *  This one writes the information to a file defined by PATH_INFO.
 */

#include <process.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <io.h>

#define TRUE 1
#define FALSE 0

char InputBuffer[4096];

/*
 *  Convert all cOld characters in cStr into cNew characters
 */
void strcvrt(char * cStr,char cOld,char cNew) {
int i;

    i = 0;
    while (cStr[i]) {
        if (cStr[i]==cOld) cStr[i] = cNew;
        i++;
    }
}

/*
 *  The string starts with two hex characters.  Return an integer formed from them.
 */
static int TwoHex2Int(char *pC) {
int Hi;
int Lo;
int Result;

    Hi = pC[0];
    if ('0'<=Hi && Hi<='9') {
        Hi -= '0';
    } else
    if ('a'<=Hi && Hi<='f') {
        Hi -= ('a'-10);
    } else
    if ('A'<=Hi && Hi<='F') {
        Hi -= ('A'-10);
    }
    Lo = pC[1];
    if ('0'<=Lo && Lo<='9') {
        Lo -= '0';
    } else
    if ('a'<=Lo && Lo<='f') {
        Lo -= ('a'-10);
    } else
    if ('A'<=Lo && Lo<='F') {
        Lo -= ('A'-10);
    }
    Result = Lo + 16*Hi;
    return Result;
}


/*
 *  Save all the CGI environment variables
 */
void StoreEnvVars(FILE *f) {
int i;

    i = 0;
    while (_environ[i]) {
        fprintf(f,"%s\r\n",_environ[i]);
        i++;
    }
}


/*
 *  Decode the given string in-place by expanding %XX escapes
 */
void urlDecode(char *p) {
char *pD;

    pD = p;
    while (*p) {
        if (*p=='%') {
            /* Escape: next 2 chars are hex representation of the actual character */
            p++;
            if (isxdigit(p[0]) && isxdigit(p[1])) {
                *pD++ = (char) TwoHex2Int(p);
                p += 2;
            }
        } else {
            *pD++ = *p++;
        }
    }
    *pD = '\0';
}


/*
 *  Return TRUE if the file is within or below the current directory.
 *  It must have a .HFO extension.
 */
static int FileNameIsSafe(char *FileName) {
char *p;

    p = FileName;
    while (*p==' '||*p=='\t') p++;
    if (*p=='\\') return FALSE;
    if (strstr(p,"..")!=NULL) return FALSE;
    if (strchr(p,':')!=NULL) return FALSE;
    if (strlen(FileName)<5) return FALSE;
    if (_stricmp(".HFO",FileName+strlen(FileName)-4)!=0) return FALSE;
    return TRUE;
}


/*
 *  Parse out and store field=value items.  Don't use strtok!
 */
void StoreField(FILE *f,char *Item) {
char *p;

    p = strchr(Item,'=');
    *p++='\0';
    urlDecode(Item);
    urlDecode(p);
    strcvrt(p,'\n',' ');
    strcvrt(p,'\r',' ');
    fprintf(f,"%s = %s\n",Item,p);
}


/*
 *  Main program
 */
int main(int argc, char *argv[]) {
int x;
int i;
char *p;
char *q;
char *FileName;
FILE *f;

    /* Turn buffering off for stdin */
    /* setvbuf(stdin,NULL,_IONBF,0); */

    /* Get the PATH_INFO */
    FileName = getenv("PATH_INFO");
    if (FileName==NULL) {

        /* Tell the client what we're going to send */
        printf("Content-type: text/html\n\n");

        /* Does not exist */
        printf("<HEAD><TITLE>Error - no filename</TITLE></HEAD>\n");
        printf("<BODY><H1>Error - no filename</H1>\n");
        printf("There is no PATH_INFO supplied.  Please modify the action parameter in your form.\n");
        printf("</BODY>\n");
        exit(0);
    }

    /* Convert PATH_INFO to a filename */
    q = FileName;
    while (*q) {
        if (*q=='/') *q = '\\';
        q++;
    }
    if (*FileName=='\\') FileName++;
    urlDecode(FileName);

    _spawnlp(_P_WAIT, "perl.exe", "perl.exe", FileName, NULL);

    return 0;
}

