# '-*- Perl -*-'
#
#
# lock.pl
#
# use the existence of a file as a semaphore lock. Based on a 
# locking mechanism I saw in the perl counter script on
# http://www.nsf.gov
#

require "flush.pl";

# unlock the lock file
#
# sample:
#
# &unlockFile(".lock");
#
sub unlockFile {
   local($SemaphoreFile) = @_;
   close (LOCK);
   unlink($SemaphoreFile);
}

# lock a file and create the filehandle LOCK
#
# Sample:
#
# if ( &lockFile(".lock", 3) ) {
#    &doSomething;
# } else {
#    print STDERR "Failed to get the lock\n";
# }
#
sub lockFile {

   local($SemaphoreFile, $Retries) = @_;

   unlink($SemaphoreFile);   # this will fail unless a process crashes,
                             # leaving the file lying around....

   $lockCount = 0;
   while (-f $SemaphoreFile) { 
      if ($lockCount > $Retries) {
         return 0; 
      }
      sleep 1;
      $lockCount++;
   }

   open(LOCK,">>$SemaphoreFile") || die "Couldn't open $SemaphoreFile\n";
   print LOCK "1";
   &flush(LOCK);
   return 1;

}

1;
