# '-*- Perl -*-'
#
# xbase.pl
#
# *perl* library for screwing around with xbase files.
#
# Copyright 1995, Brian C. Jepson
#                 Director of Advanced Technology,
#                 Prosoft Systems International, Inc.
#                 (bjepson@conan.ids.net)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Sample usage:
#
# require "xbase.pl";
#
# # the insert array
# $insert{'FIELD2'} = "sample data";
# $insert{'T'} = 1.26353;
#
# &xbHeader("test1.bak", *fields, *field_def);
# &xbInsert("test1.bak", *fields, *field_def, *insert);

require "lock.pl";
require "flush.pl";

$FLD_TYPE = 1;
$FLD_POSITION = 2;
$FLD_LENGTH = 3;
$FLD_PRECISION = 4;

sub xbInsert {

    local($dbf, *fields, *field_def, *insert) = @_;
    local($type, $len, $prec);

    if ( &lockFile($dbf . "test.lock", 10) ) {

	open (dbf, "+<$dbf") || (print "File $dbf could not be opened" && die);
	binmode(dbf);

	select(dbf); $| = 1;		# make it unbuffered
	select(STDOUT);

	# print the record separator
	$output =  " ";

	# loop through the list of fields. If we have a value for them,
	# they will be in the associative array $insert{}. If they're
	# not, *perl* will make them blank or zero
	#
	foreach $t (@fields) {

	    $type = $field_def{$t, $FLD_TYPE}; 
	    $len =  $field_def{$t, $FLD_LENGTH}; 
	    $prec = $field_def{$t, $FLD_PRECISION};

	    $output .= &xbWriteFld($insert{$t}, $type, $len, $prec);
	
	}

	# print the eof marker
	$output .= "\x1A";

	&flush(LOCK);
	seek(LOCK, 0, 2);
	if (tell(LOCK) == 1) {

	    # position at end of file and write new record
	    seek(dbf, 0, 2);
	    print dbf $output;

	    # determine the record count
	    seek(dbf, 4, 0);
	    read(dbf, $count, 4);
	    # write out the new record count.
	    seek(dbf, 4, 0);
            $cnt= unpack("I*", $count);
            print dbf pack("I*", ++$cnt);
	    $retval = 1;
	}
	else {
	    $retval = 0;
	}
	close(dbf);
    } else {
	$retval = 0;
    }

    &unlockFile($dbf . "test.lock");

    return $retval;
}



#
# xbWriteFld
#
#
sub xbWriteFld {

    local($value, $type, $len, $precision) = @_;
    local($retval);

    if ($type eq "C") {
	$retval = sprintf("%-${len}.${len}s", $value);
    } elsif ($type eq "N") {
	$retval = sprintf("%${len}.${precision}f", $value);
    } elsif ($type eq "L") {

    } elsif ($type eq "D") {

    } else {

    }

}


#
# xbHeader
#
# pull the dbf header into some arrays.
#
# based on an example by David_Rensin@mail.amsinc.com
# (ftp://ftp.cis.ufl.edu/pub/perl/db/scripts/xbase/read_dbf.pl.gz)
#
# Sample code to read in and print out field definitions from a dbf:
#
# %fields = ();
# %field_def = ();
# 
# &xbHeader("test1.dbf", *fields, *field_def);
#
# foreach $t (@fields) {
#    printf "$t $field_def{$t, $FLD_TYPE}, $field_def{$t, $FLD_POSITION}, 
#             $field_def{$t, $FLD_LENGTH}\n";
# }
#
sub xbHeader {

    local ($fileName, *fields,  *field_def) = @_;

    open (dbf, $fileName) ||  (print "File $dbf could not be opened" && die);
    binmode(dbf);

    read(dbf,$foo,32);

    # read the first field definition from the dbf file
    read(dbf,$foo,32);

    # read a line as long as we keep finding field names followed by types
    #
    $field_num = 0;
    while ($foo =~ /^\w/ && $foo !~ /^. /) {

	$field_name = substr($foo, 0, 10);
	if (index($field_name, "\0", 0)) {
	    $field_name = substr($field_name, 0, index($field_name, "\0", 0));
	}

	$fields[$field_num++] = $field_name;

	$field_def{$field_name, $FLD_TYPE} = substr($foo,11,1);

	$junk = substr($foo,12,1);
	$field_def{$field_name, $FLD_POSITION} = unpack("C",$junk);

	$junk = substr($foo,16,1);
	$field_def{$field_name, $FLD_LENGTH} = unpack("C",$junk);

	$junk = substr($foo,17,1);
	$field_def{$field_name, $FLD_PRECISION} = unpack("C",$junk);


	read(dbf,$foo,32);	# next field definition
    }

    close(dbf);
}



#
# xbGoTop
#
# based on an example by David_Rensin@mail.amsinc.com
# (ftp://ftp.cis.ufl.edu/pub/perl/db/scripts/xbase/read_dbf.pl.gz)
#
# This procedure will go to the top of a dbf file
# It expects that the file handle dbf is associated with a dbf file
#
sub xbGoTop {

    seek(dbf, 0, 0);		# start at the very top

    read(dbf,$foo,32);
    read(dbf,$foo,32);

    # read a line as long as we keep finding field names followed by types
    #
    while ($foo =~ /^\w/ && $foo !~ /^. /) {

	read(dbf,$foo,32);	# next field definition
    }
    seek(dbf, -32, 1);
    while ($x = getc(dbf) ne " ") {
    	print $x, "\n";
    }
    seek(dbf, -1, 1);
#    read(dbf, $foo, 232);
    # now we are at the top

}




sub xbFind {

    local($seekval, $seekkey, *record) = @_;
    local($recstart, $offset, $junk, $t, $retval, $reclen);

    $retval = 0;
    &xbGoTop;

    foreach $t (@fields) {
	$reclen = $field_def{$t, $FLD_POSITION} + $field_def{$t, $FLD_LENGTH};
    }

    # loop thru the table
    while (!eof(dbf)) {

	# remember the byte offset of this record
	$recstart = tell(dbf);

	# seek the offset of the $seekkey
	seek(dbf, $field_def{$seekkey, $FLD_POSITION}, 1);

	# read its value in
	read(dbf, $junk, $field_def{$seekkey, $FLD_LENGTH});

	# if a non-exact search is successful, store the record
	# to the $record{} array, and exit the loop
	#
	if (substr($junk, 0, length($seekval)) eq $seekval) {

	    # reposition the record pointer on the record's start
	    seek(dbf, $recstart, 0);

	    # skip past the deleted marker
	    seek(dbf, 1, 1);

	    # load each field into the array
	    # $fields should be defined using &dbfHeader
	    #
	    foreach $t (@fields) {

		read(dbf, $junk, $field_def{$t, $FLD_LENGTH});
		$record{$t} = $junk;

	    }
	    $retval = 1;
	    last;
	}
	seek(dbf, $recstart + $reclen, 0);
    }
    return $retval;
}


#
# generate an associative array of distinct occurrances
# of a key along with a count.
#
sub xbCount {

    local($seekkey, *totals) = @_;
    local($recstart, $offset, $junk, $t, $reclen);

    &xbGoTop;
    print eof(dbf), "\n";
    foreach $t (@fields) {
	$reclen = $field_def{$t, $FLD_POSITION} + $field_def{$t, $FLD_LENGTH};
    }

    # loop thru the table
    while (!eof(dbf)) {

	# remember the byte offset of this record
	$recstart = tell(dbf);

	# seek the offset of the $seekkey
	seek(dbf, $field_def{$seekkey, $FLD_POSITION}, 1);

	# read its value in
	read(dbf, $junk, $field_def{$seekkey, $FLD_LENGTH});

	if ($junk ne "") {
	    $totals{$junk}++;
	}

	seek(dbf, $recstart + $reclen, 0);
    }
    return 1;
}

1;

